@@ -121,7 +121,7 @@ use strict;
 use POSIX qw/locale_h/;
 use locale;
 use vars qw ($VERSION @ISA @REGEXS);
-$VERSION = 0.82;
+$VERSION = 0.88;
 
 use overload (
     '""'       => \&stringify,
@@ -253,7 +253,7 @@ dotted_decimal_version:
 		}
 		$j = 0;
 	    }
-	
+
 	    if ($strict && $i < 2) {
 		# requires v1.2.3
 		return BADVERSION($s,$errstr,"Invalid version format (dotted-decimal versions require at least three parts)");
@@ -416,7 +416,7 @@ sub scan_version {
     if ( !$qv && $width < 3 ) {
 	$$rv->{width} = $width;
     }
-    
+
     while (isDIGIT($pos)) {
 	$pos++;
     }
@@ -441,7 +441,7 @@ sub scan_version {
 			$orev = $rev;
  			$rev += $s * $mult;
  			$mult /= 10;
-			if (   (abs($orev) > abs($rev)) 
+			if (   (abs($orev) > abs($rev))
 			    || (abs($rev) > $VERSION_MAX )) {
 			    warn("Integer overflow in version %d",
 					   $VERSION_MAX);
@@ -460,7 +460,7 @@ sub scan_version {
 			$orev = $rev;
  			$rev += $end * $mult;
  			$mult *= 10;
-			if (   (abs($orev) > abs($rev)) 
+			if (   (abs($orev) > abs($rev))
 			    || (abs($rev) > $VERSION_MAX )) {
 			    warn("Integer overflow in version");
 			    $end = $s - 1;
@@ -468,7 +468,7 @@ sub scan_version {
 			    $vinf = 1;
 			}
  		    }
- 		} 
+ 		}
   	    }
 
   	    # Append revision
@@ -517,7 +517,7 @@ sub scan_version {
 	#  gcc version 3.3 20030304 (Apple Computer, Inc. build 1640)
 	#  for ( len = 2 - len; len > 0; len-- )
 	#  av_push(MUTABLE_AV(sv), newSViv(0));
-	# 
+	#
 	$len = 2 - $len;
 	while ($len-- > 0) {
 	    push @av, 0;
@@ -557,7 +557,7 @@ sub new
 	my ($class, $value) = @_;
 	my $self = bless ({}, ref ($class) || $class);
 	my $qv = FALSE;
-	
+
 	if ( ref($value) && eval('$value->isa("version")') ) {
 	    # Can copy the elements directly
 	    $self->{version} = [ @{$value->{version} } ];
@@ -596,7 +596,7 @@ sub new
 	    $value = sprintf("%.9f",$value);
 	    $value =~ s/(0+)$//; # trim trailing zeros
 	}
-	
+
 	my $s = scan_version($value, \$self, $qv);
 
 	if ($s) { # must be something left over
@@ -609,7 +609,7 @@ sub new
 
 *parse = \&new;
 
-sub numify 
+sub numify
 {
     my ($self) = @_;
     unless (_verify($self)) {
@@ -650,7 +650,7 @@ sub numify
     return $string;
 }
 
-sub normal 
+sub normal
 {
     my ($self) = @_;
     unless (_verify($self)) {
@@ -693,9 +693,9 @@ sub stringify
 	require Carp;
 	Carp::croak("Invalid version object");
     }
-    return exists $self->{original} 
-    	? $self->{original} 
-	: exists $self->{qv} 
+    return exists $self->{original}
+    	? $self->{original}
+	: exists $self->{qv}
 	    ? $self->normal
 	    : $self->numify;
 }
@@ -733,8 +733,8 @@ sub vcmp
     }
 
     # tiebreaker for alpha with identical terms
-    if ( $retval == 0 
-	&& $l == $r 
+    if ( $retval == 0
+	&& $l == $r
 	&& $left->{version}[$m] == $right->{version}[$m]
 	&& ( $lalpha || $ralpha ) ) {
 
@@ -766,7 +766,7 @@ sub vcmp
 	}
     }
 
-    return $retval;  
+    return $retval;
 }
 
 sub vbool {
@@ -774,8 +774,8 @@ sub vbool {
     return vcmp($self,$self->new("0"),1);
 }
 
-sub vnoop { 
-    require Carp; 
+sub vnoop {
+    require Carp;
     Carp::croak("operation not supported with version object");
 }
 
@@ -819,25 +819,57 @@ sub _verify {
     }
 }
 
+sub _is_non_alphanumeric {
+    my $s = shift;
+    $s = new charstar $s;
+    while ($s) {
+	return 0 if isSPACE($s); # early out
+	return 1 unless (isALPHA($s) || isDIGIT($s) || $s =~ /[.-]/);
+	$s++;
+    }
+    return 0;
+}
+
 sub _un_vstring {
     my $value = shift;
     # may be a v-string
-    if ( $] >= 5.006_000 && length($value) >= 3 && $value !~ /[._]/) {
-	foreach my $char (split(//,$value)) {
-	    # if one of the characters is non-text assume v-string
-	    if (ord($char) < ord(" ")) {
-		my $tvalue = sprintf("v%vd",$value);
-		if ( $tvalue =~ /^v\d+(\.\d+){2,}$/ ) {
-		    # must be a v-string
-		    $value = $tvalue;
-		}
-		last;
+    if ( length($value) >= 3 && $value !~ /[._]/
+	&& _is_non_alphanumeric($value)) {
+	my $tvalue;
+	if ( $] ge 5.008_001 ) {
+	    $tvalue = _find_magic_vstring($value);
+	    $value = $tvalue if length $tvalue;
+	}
+	elsif ( $] ge 5.006_000 ) {
+	    $tvalue = sprintf("v%vd",$value);
+	    if ( $tvalue =~ /^v\d+(\.\d+){2,}$/ ) {
+		# must be a v-string
+		$value = $tvalue;
 	    }
 	}
     }
     return $value;
 }
 
+sub _find_magic_vstring {
+    my $value = shift;
+    my $tvalue = '';
+    require B;
+    my $sv = B::svref_2object(\$value);
+    my $magic = ref($sv) eq 'B::PVMG' ? $sv->MAGIC : undef;
+    while ( $magic ) {
+	if ( $magic->TYPE eq 'V' ) {
+	    $tvalue = $magic->PTR;
+	    $tvalue =~ s/^v?(.+)$/v$1/;
+	    last;
+	}
+	else {
+	    $magic = $magic->MOREMAGIC;
+	}
+    }
+    return $tvalue;
+}
+
 sub _VERSION {
     my ($obj, $req) = @_;
     my $class = ref($obj) || $obj;
@@ -859,7 +891,7 @@ sub _VERSION {
     if ( defined $req ) {
 	unless ( defined $version ) {
 	    require Carp;
-	    my $msg =  $] < 5.006 
+	    my $msg =  $] < 5.006
 	    ? "$class version $req required--this is only version "
 	    : "$class does not define \$$class\::VERSION"
 	      ."--version check failed";
@@ -877,14 +909,14 @@ sub _VERSION {
 	if ( $req > $version ) {
 	    require Carp;
 	    if ( $req->is_qv ) {
-		Carp::croak( 
+		Carp::croak(
 		    sprintf ("%s version %s required--".
 			"this is only version %s", $class,
 			$req->normal, $version->normal)
 		);
 	    }
 	    else {
-		Carp::croak( 
+		Carp::croak(
 		    sprintf ("%s version %s required--".
 			"this is only version %s", $class,
 			$req->stringify, $version->stringify)
